/*
  ==============================================================================

    ResizableImage.cpp
    Created: 31 Mar 2016 11:36:17am
    Author:  George Scrivener

  ==============================================================================
*/

#include "ResizableImage.h"

static int iInstanceCount1 = 0;
static int iInstanceCount2 = 0;
//MARK: - Insets
// ============================== Insets ==============================
ResizableImage::ImageInsets::ImageInsets()
:   ResizableImage::ImageInsets(0, 0, 0, 0)
{
}

ResizableImage::ImageInsets::ImageInsets(int leftInset_,
                                         int rightInset_,
                                         int topInset_,
                                         int bottomInset_,
                                         int overscalingFactor_)
:   leftInset(leftInset_), rightInset(rightInset_), topInset(topInset_), bottomInset(bottomInset_), overscalingFactor(overscalingFactor_)
{
    jassert(leftInset == 0 || leftInset % overscalingFactor == 0);
    jassert(rightInset == 0 || rightInset % overscalingFactor == 0);
    jassert(topInset == 0 || topInset % overscalingFactor == 0);
    jassert(bottomInset == 0 || bottomInset % overscalingFactor == 0);
}

bool ResizableImage::ImageInsets::isZero() const
{
    return leftInset   == 0
        && rightInset  == 0
        && topInset    == 0
        && bottomInset == 0;
    
}

bool ResizableImage::ImageInsets::operator==(const ResizableImage::ImageInsets& other) const
{
    return leftInset == other.leftInset
        && rightInset == other.rightInset
        && topInset == other.topInset
        && bottomInset == other.bottomInset
        && overscalingFactor == other.overscalingFactor;
}

bool ResizableImage::ImageInsets::operator!=(const ResizableImage::ImageInsets& other) const
{
    return !(*this == other);
}


//MARK: - ResizableImage
// ============================== ResizableImage ==============================
ResizableImage::ResizableImage()
:   _imageInsets(0, 0, 0, 0)
{
    iInstanceCount1++;
    Logger::outputDebugString("Bapuji - ResizableImage Created " + String(iInstanceCount1));
}

ResizableImage::~ResizableImage()
{
    iInstanceCount2++;
    Logger::outputDebugString("Bapuji - ResizableImage Deleted " + String(iInstanceCount2));
}

ResizableImage& ResizableImage::operator= (const ResizableImage& other)
{
    if(other._image.isValid())
    {
        _image = other._image.createCopy();
        createAllImages();
        _imageInsets = other._imageInsets;
    }
    return *this;
}

ResizableImage::ResizableImage(const juce::Image& image, ImageInsets insets)
:   //_image(image),
    _imageInsets(insets)
{
    _image = image.createCopy();
    createAllImages();
    iInstanceCount1++;
    Logger::outputDebugString("Bapuji - ResizableImage Created " + String(iInstanceCount1));
    jassert(image.getWidth() >= insets.leftInset + insets.rightInset);
    jassert(image.getHeight() >= insets.topInset + insets.bottomInset);
}

ResizableImage::ResizableImage(const char * data, int dataSize,
                               int leftInset, int rightInset, int topInset, int bottomInset,
                               int overscalingFactor)
:   ResizableImage(ImageFileFormat::loadFrom(data, dataSize),
                   ImageInsets(leftInset, rightInset, topInset, bottomInset, overscalingFactor))
{
}

bool ResizableImage::operator==(const ResizableImage& other) const
{
    return _image == other._image && _imageInsets == other._imageInsets;
}

bool ResizableImage::operator!=(const ResizableImage& other) const
{
    return (*this == other);
}

bool ResizableImage::isValid() const
{
    return _image.isValid();
}

void ResizableImage::createAllImages()
{
    if(_image.isValid())
    {
        _imageTop = _image.createCopy();
        _imageBottom = _image.createCopy();
        _imageLeft = _image.createCopy();
        _imageRight = _image.createCopy();
        _imageTopLeft = _image.createCopy();
        _imageTopRight = _image.createCopy();
        _imageBottomLeft = _image.createCopy();
        _imageBottomRight = _image.createCopy();
    }
}

void ResizableImage::setImage(const juce::Image& image, ResizableImage::ImageInsets insets)
{
    _image = image.createCopy();
    createAllImages();
    _imageInsets = insets;
    jassert(image.getWidth() >= insets.leftInset + insets.rightInset);
    jassert(image.getHeight() >= insets.topInset + insets.bottomInset);
}

void ResizableImage::setImage(const char *data, int dataSize, ResizableImage::ImageInsets insets)
{
    //auto image = ImageFileFormat::loadFrom(data, dataSize);
    setImage(ImageFileFormat::loadFrom(data, dataSize), insets);
}

void ResizableImage::setImage(const char *data, int dataSize, int leftInset, int rightInset, int topInset, int bottominset)
{
    ImageInsets insets(leftInset, rightInset, topInset, bottominset);
    setImage(data, dataSize, insets);
}

int ResizableImage::imageHeight() const
{
    return _image.getHeight();
}

int ResizableImage::imageWidth() const
{
    return _image.getWidth();
}

const ResizableImage::ImageInsets& ResizableImage::insets() const
{
    return _imageInsets;
}

void ResizableImage::drawInRect(const juce::Graphics &g, Rectangle<int> rect) const
{
    drawInRect(g, rect.getX(), rect.getY(), rect.getWidth(), rect.getHeight());
}

void ResizableImage::drawInRect(const juce::Graphics &g, int x, int y, int width, int height) const
{
    //return; //Added by Bapuji for Testing
    //jassert(_image.isValid());
    
    if (!(width > 0 && height > 0) || (!_image.isValid())) { return; }
    
    if (resizeMode == ResizeMode::DontResize || _imageInsets.isZero()) {
        // Draw image as normal and return
        g.drawImageWithin(_image,
                          x, y, width, height,
                          RectanglePlacement::stretchToFit);
        return;
    }
    
    int leftInset = _imageInsets.leftInset / _imageInsets.overscalingFactor;
    int rightInset = _imageInsets.rightInset / _imageInsets.overscalingFactor;
     int topInset = _imageInsets.topInset / _imageInsets.overscalingFactor;
     int bottomInset = _imageInsets.bottomInset / _imageInsets.overscalingFactor;
    
     int imageWidth = _image.getWidth();
     int imageHeight = _image.getHeight();
     int centreWidth = width - leftInset - rightInset;
     int centreHeight = height - topInset - bottomInset;
    
     int imageCentreWidth = imageWidth - _imageInsets.leftInset - _imageInsets.rightInset;
     int imageCentreHeight = imageHeight - _imageInsets.topInset - _imageInsets.bottomInset;
    
    // ============================== Draw Centre ==============================
    if (centreWidth > 0 && centreHeight > 0 && imageCentreWidth > 0 && imageCentreHeight > 0) {
        switch (resizeMode) {
            case OnlyDrawOutside:
                break;
                
            case StretchInsde:
                g.drawImage(_image,
                            limitX(x + leftInset),
                            limitY(y + topInset),
                            limitWidth(centreWidth),
                            limitHeight(centreHeight),
                            limitX(_imageInsets.leftInset),
                            limitY(_imageInsets.topInset),
                            limitWidth(imageCentreWidth),
                            limitHeight(imageCentreHeight));
                
//                            g.drawImage(_image,
//                                        0,
//                                        0,
//                                        width,
//                                        height,
//                                        0,
//                                        0,
//                                        _image.getWidth(),
//                                        _image.getHeight());
                break;
                
            case DontResize:
                /// This case should have already been handled and should have returned
                jassertfalse;
                break;
        }
    }
    
    // ============================== Draw Edges ==============================
    
    if (centreWidth > 0) {
        // Top
        g.drawImage(_imageTop,
                    limitX(x + rightInset),
                    limitY(y),
                    limitWidth(centreWidth),
                    limitHeight(topInset),
                    limitX(_imageInsets.leftInset),
                           limitX(0),
                                  limitWidth(imageCentreWidth),
                                  limitHeight(_imageInsets.topInset));
        
        // Bottom
        g.drawImage(_imageBottom,
                    limitX(x + leftInset),
                    limitY(y + height - bottomInset),
                    limitWidth(centreWidth),
                    limitHeight(bottomInset),
                    limitX(_imageInsets.leftInset),
                    limitY(imageHeight - _imageInsets.bottomInset),
                    limitWidth(imageCentreWidth),
                    limitHeight(_imageInsets.bottomInset));
    }

    if (centreHeight > 0) {
        // Right
        g.drawImage(_imageRight,
                    limitX(x + width - rightInset),
                    limitY(y + topInset),
                    limitWidth(rightInset),
                    limitHeight(centreHeight),
                    limitX(imageWidth - _imageInsets.rightInset),
                    limitY(_imageInsets.topInset),
                    limitWidth(_imageInsets.rightInset),
                    limitHeight(imageCentreHeight));

        // Left
        g.drawImage(_imageLeft,
                    limitX(x),
                    limitY(y + topInset),
                    limitWidth(leftInset),
                    limitHeight(centreHeight),
                    limitX(0),
                    limitY(_imageInsets.topInset),
                    limitWidth(_imageInsets.leftInset),
                    limitHeight(imageCentreHeight));
    }
    
    // ============================== Draw Corners ==============================
    // Top Left
    g.drawImage(_imageTopLeft,
                limitX(x),
                limitY(y),
                limitWidth(leftInset),
                limitHeight(topInset),
                limitX(0),
                limitY(0),
                limitWidth(_imageInsets.leftInset),
                limitHeight(_imageInsets.topInset));
    
    // Top Right
    g.drawImage(_imageTopRight,
                limitX(x + width - rightInset),
                limitY(y),
                limitWidth(rightInset),
                limitHeight(topInset),
                limitX(imageWidth - _imageInsets.rightInset),
                limitY(0),
                limitWidth(_imageInsets.rightInset),
                limitHeight(_imageInsets.topInset));
    
    // Bottom Left
    g.drawImage(_imageBottomLeft,
                limitX(x),
                limitY(y + height - bottomInset),
                limitWidth(leftInset),
                limitHeight(bottomInset),
                limitX(0),
                limitY(imageHeight - _imageInsets.bottomInset),
                limitWidth(_imageInsets.leftInset),
                limitHeight(_imageInsets.bottomInset));
    
    // Bottom Right
    g.drawImage(_imageBottomRight,
                limitX(x + width - rightInset),
                limitY(y + height - bottomInset),
                limitWidth(rightInset),
                limitHeight(bottomInset),
                limitX(imageWidth - _imageInsets.rightInset),
                limitY(imageHeight - _imageInsets.bottomInset),
                limitWidth(_imageInsets.rightInset),
                limitHeight(_imageInsets.bottomInset));
}

int ResizableImage::limitX(const int iNumberToLimit) const
{
    int iNewNumber = iNumberToLimit;
//    if(iNewNumber < 0)
//    {
//        iNewNumber = 0;
//    }
//    else if(iNewNumber > 20)
//    {
//        iNewNumber = 20;
//    }
    
    return iNewNumber;
}

int ResizableImage::limitY(const int iNumberToLimit) const
{
    int iNewNumber = iNumberToLimit;
//    if(iNewNumber < 0)
//    {
//        iNewNumber = 0;
//    }
//    else if(iNewNumber > 20)
//    {
//        iNewNumber = 20;
//    }
    
    return iNewNumber;
}

int ResizableImage::limitWidth(const int iNumberToLimit) const
{
    int iNewNumber = iNumberToLimit;
//    if(iNewNumber < 10)
//    {
//        iNewNumber = 10;
//    }
//    else if(iNewNumber > 200)
//    {
//        iNewNumber = 200;
//    }
    
    return iNewNumber;
}

int ResizableImage::limitHeight(const int iNumberToLimit) const
{
    int iNewNumber = iNumberToLimit;
//    if(iNewNumber < 10)
//    {
//        iNewNumber = 10;
//    }
//    else if(iNewNumber > 190)
//    {
//        iNewNumber = 190;
//    }
    
    return iNewNumber;
}
