
#include "ResizableImageLabel.h"

// ============================== Insets ==============================
ResizableImage::ImageInsets::ImageInsets()
:   ResizableImage::ImageInsets(0, 0, 0, 0)
{
}

ResizableImage::ImageInsets::ImageInsets(int leftInset_,
                                         int rightInset_,
                                         int topInset_,
                                         int bottomInset_,
                                         int overscalingFactor_)
:   leftInset(leftInset_), rightInset(rightInset_), topInset(topInset_), bottomInset(bottomInset_), overscalingFactor(overscalingFactor_)
{
    jassert(leftInset == 0 || leftInset % overscalingFactor == 0);
    jassert(rightInset == 0 || rightInset % overscalingFactor == 0);
    jassert(topInset == 0 || topInset % overscalingFactor == 0);
    jassert(bottomInset == 0 || bottomInset % overscalingFactor == 0);
}

bool ResizableImage::ImageInsets::isZero() const
{
    return leftInset   == 0
        && rightInset  == 0
        && topInset    == 0
        && bottomInset == 0;
    
}

// ============================== ResizableImage ==============================
ResizableImage::ResizableImage()
:   _imageInsets(0, 0, 0, 0)
{
}

ResizableImage::~ResizableImage()
{
}


void ResizableImage::setImage(const juce::Image& image, ResizableImage::ImageInsets insets)
{
    _image = image.createCopy();
    _imageInsets = insets;
    jassert(image.getWidth() >= insets.leftInset + insets.rightInset);
    jassert(image.getHeight() >= insets.topInset + insets.bottomInset);
}

void ResizableImage::drawInRect(const juce::Graphics &g, Rectangle<int> rect) const
{
    drawInRect(g, rect.getX(), rect.getY(), rect.getWidth(), rect.getHeight());
}

void ResizableImage::drawInRect(const juce::Graphics &g, int x, int y, int width, int height) const
{
    jassert(_image.isValid());
    
    if (!(width > 0 && height > 0))
    {
        return;
    }
    
    if (resizeMode == ResizeMode::DontResize || _imageInsets.isZero())
    {
        // Draw image as normal and return
        g.drawImageWithin(_image,
                          x, y, width, height,
                          RectanglePlacement::stretchToFit);
        return;
    }
    
    int leftInset = _imageInsets.leftInset / _imageInsets.overscalingFactor;
    int rightInset = _imageInsets.rightInset / _imageInsets.overscalingFactor;
    int topInset = _imageInsets.topInset / _imageInsets.overscalingFactor;
    int bottomInset = _imageInsets.bottomInset / _imageInsets.overscalingFactor;
    
    int imageWidth = _image.getWidth();
    int imageHeight = _image.getHeight();
    int centreWidth = width - leftInset - rightInset;
    int centreHeight = height - topInset - bottomInset;
    
    int imageCentreWidth = imageWidth - _imageInsets.leftInset - _imageInsets.rightInset;
    int imageCentreHeight = imageHeight - _imageInsets.topInset - _imageInsets.bottomInset;
    
    // ============================== Draw Centre ==============================
    if (centreWidth > 0 && centreHeight > 0)
    {
        switch (resizeMode)
        {
            case OnlyDrawOutside:
                break;
                
            case StretchInsde:
                g.drawImage(_image,
                            x + leftInset,
                            y + topInset,
                            centreWidth,
                            centreHeight,
                            _imageInsets.leftInset,
                            _imageInsets.topInset,
                            imageCentreWidth,
                            imageCentreHeight);
                break;
        }
    }
    
    // ============================== Draw Edges ==============================
    
    if (centreWidth > 0)
    {
        // Top
        g.drawImage(_image,
                    x + rightInset,
                    y,
                    centreWidth,
                    topInset,
                    _imageInsets.leftInset,
                    0,
                    imageCentreWidth,
                    _imageInsets.topInset);
        
        // Bottom
        g.drawImage(_image,
                    x + leftInset,
                    y + height - bottomInset,
                    centreWidth,
                    bottomInset,
                    _imageInsets.leftInset,
                    imageHeight - _imageInsets.bottomInset,
                    imageCentreWidth,
                    _imageInsets.bottomInset);
    }
    
    if (centreHeight > 0)
    {
        // Right
        g.drawImage(_image,
                    x + width - rightInset,
                    y + topInset,
                    rightInset,
                    centreHeight,
                    imageWidth - _imageInsets.rightInset,
                    _imageInsets.topInset,
                    _imageInsets.rightInset,
                    imageCentreHeight);
        
        // Left
        g.drawImage(_image,
                    x,
                    y + topInset,
                    leftInset,
                    centreHeight,
                    0,
                    _imageInsets.topInset,
                    _imageInsets.leftInset,
                    imageCentreHeight);
    }
    
    // ============================== Draw Corners ==============================
    // Top Left
    g.drawImage(_image,
                x,
                y,
                leftInset,
                topInset,
                0,
                0,
                _imageInsets.leftInset,
                _imageInsets.topInset);
    
    // Top Right
    g.drawImage(_image,
                x + width - rightInset,
                y,
                rightInset,
                topInset,
                imageWidth - _imageInsets.rightInset,
                0,
                _imageInsets.rightInset,
                _imageInsets.topInset);
    
    // Bottom Left
    g.drawImage(_image,
                x,
                y + height - bottomInset,
                leftInset,
                bottomInset,
                0,
                imageHeight - _imageInsets.bottomInset,
                _imageInsets.leftInset,
                _imageInsets.bottomInset);
    
    // Bottom Right
    g.drawImage(_image,
                x + width - rightInset,
                y + height - bottomInset,
                rightInset,
                bottomInset,
                imageWidth - _imageInsets.rightInset,
                imageHeight - _imageInsets.bottomInset,
                _imageInsets.rightInset,
                _imageInsets.bottomInset);
}
